/* Copyright (c) 2018-2020, The Linux Foundation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *     * Neither the name of The Linux Foundation nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/* qsap interface for hostapd daemon */

#include <unistd.h>
#include <net/if.h>
#include <sys/socket.h>	
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <linux/wireless.h>

#include<android-base/logging.h>
#include <android/log.h>

#define LOG_TAG "qsap_handler"

#ifndef ALOG
#define ALOG(priority, tag, ...) __android_log_print(ANDROID_##priority, tag, __VA_ARGS__)
#endif
#ifndef ALOGD
#define ALOGD(...) __android_log_print(ANDROID_LOG_DEBUG, LOG_TAG, __VA_ARGS__)
#endif
#ifndef ALOGE
#define ALOGE(...) __android_log_print(ANDROID_LOG_ERROR, LOG_TAG, __VA_ARGS__)	
#endif

#include "string"
#include "qsap_handler.h"
#include <qsap_api.h>

namespace vendor {
namespace qti {
namespace hardware {
namespace wifi {
namespace hostapd {
namespace V1_2 {
namespace implementation {
namespace qsap_handler {

int run_qsap_cmd(int argc, char** argv) {
	int ret = 0;

	if (argc < 3)
		return -1;

	if (!strcmp(argv[1], "qccmd")) {
		ret = qsap_hostd_exec(argc, argv);
	} else if (!strcmp(argv[1], "create") &&
		qsap_add_or_remove_interface(argv[2], 1)) {
	} else if (!strcmp(argv[1], "remove") &&
		qsap_add_or_remove_interface(argv[2], 0)) {
	} else if (!strcmp(argv[1], "bridge")) {
		ret = qsap_control_bridge(argc, argv);
	} else if (!strcmp(argv[1], "setsoftap")) {
		ret = qsapsetSoftap(argc, argv);
	} else {
		ret = -1;
	}

	return ret;
}


/* WiFi_2798_A S */
std::string setMaxAssoc(int argc, char **argv)
{
    int sock = socket ( AF_INET , SOCK_DGRAM , 0 );
    if (sock < 0) {
        ALOGE("Softap setMaxAssoc - failed to open socket");
        return "failed";
    }
    if (argc < 3) {
        ALOGE("Softap setMaxAssoc - missing arguments");
        close(sock);
        return"failed"; 
    }
    struct iwreq wrq;
    int temp = 0;
    int ret = 0 ; 
    sscanf(argv[2], "%i", &temp);
    memset(&wrq, 0, sizeof(wrq));
    strncpy(wrq.ifr_name, "wlan0", sizeof(wrq.ifr_name));
    wrq.u.mode = 1;
    memcpy(wrq.u.name + sizeof(__u32), &temp, sizeof(temp));
    ret = ioctl(sock, SIOCIWFIRSTPRIV, &wrq);
    ALOGD("Softap setMaxAssoc %d ret:%d", temp, ret);
    close(sock);
    if ( ret == 0 ) {
        return "success";
    }else{
        return std::to_string(ret);
    }
}

std::string qcSoftAP_exec(int argc, char ** argv, char *respBuf,int respbufSize)
{
    char qcCmdBuf[MAX_CMD_SIZE], *pCmdBuf;
    u32 len = MAX_CMD_SIZE;
    int i = 2 , ret ;

    if ( argc < 4 ) {
        ALOGD("failure: invalid arguments");
        return "failed";
    }

    argc -= 2;
    pCmdBuf = qcCmdBuf;

    while (argc--) {
        ret = snprintf(pCmdBuf, len, " %s", argv[i]);
        if ((ret < 0) || (ret >= (int)len)) {
            /* Error case */
            /* TODO: Command too long send the error message */
            *pCmdBuf = '\0';
             break;
        }
        ALOGD("argv[%d] (%s)",i, argv[i]);
        pCmdBuf= pCmdBuf + ret ;
        len = len - ret ;
        i++;
    }

    memset(respBuf, 0, respbufSize);
    len = respbufSize ;
    qsap_hostd_exec_cmd(qcCmdBuf, respBuf, (u32*)&len);

    if (len >= (u32)respbufSize) {
        len = respbufSize - 1 ;
    }
    respBuf [ len ] = '\0' ;

    ALOGD("QCCMD data (%s) resp=%s", qcCmdBuf, respBuf);
    return std::string(respBuf);
}
/* WiFi_2798_A E */
}  // namespace qsap_handler
}  // namespace implementation
}  // namespace V1_2
}  // namespace hostapd
}  // namespace wifi
}  // namespace hardware
}  // namespace qti
}  // namespace vendor
